/*
 * smi/appConf/ApplicationConfiguration.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.appConf;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import help.Help;
import ssonet.SSONETConstants;
import ssonet.SSONETConstraints;
/**
* With the help of this class an application programmer can show a dialog
* which configures single actions.
* <image src=./myimages/actionconfiguration.gif>
* The user can set ratings for the security goals anonymity, confidentiality,
* integrity and accountability.
* The settings will be saved in a database, so that they can be used later
* during negotiations.
*/

final public class ApplicationConfiguration extends JDialog implements ActionListener {
/** Contains the configuration of actions during program execution
* Access indices are the action numbers.
* Registered objects are action configurations.
* @see ActionConfiguration
    */
    private Hashtable configuration;

    /** action settings to configure
    */
    private ActionConfiguration actionConfiguration;

    /** The dialog is divided into 4 panels.
    * The following variables contain the panels for the security goals.
    */
    private ApplicationConfigurationPanel panelAccountability;
    private ApplicationConfigurationPanel panelAnonymity;
    private ApplicationConfigurationPanel panelIntegrity;
    private ApplicationConfigurationPanel panelConfidentiality;

    /** Contains the name of the application
    * The configuration data will be saved in a file with the name <applicationName>.cfg
    */
    private String applicationName;

    /** Buttons for some actions
    */
    private JButton okButton, helpButton, defaultButton;

    /** Unique id of the action which is to be configured
    */
    private int actionID;

    /** File to save the configuration data
    */
    private File configurationFile;

    private static long interactiveTime=0;

    ActionConfiguration defaultValues=null;
    /** Ascertains the configuration file with the help of the application
    * name and the system property ssonet_cfg.
    * ssonet_cfg has to contain the directory where
    * the configuration data should be saved
    * @param applicationName name of the application
    * for which the action should be configured
    * It is neccessary for the determination of
    * the filename <applicationName>.cfg
    * @return configuration file
    * null, if ssonet_cfg is not set
    * Hint: The configuration file doesnt have to exist.
    * Merely a file object is created.
    */
    static private File getConfigurationFile(String applicationName) {
	Object object = System.getProperty("ssonet_cfg");
	// is ssonet_cfg existent and a String?
	if((object == null) || !(object instanceof String))
	    return null;
	// create object of type File (according to ssonet_cfg and applicationName)
	return new File((String)object, applicationName + ".cfg");
    }

    /** Reads the whole configuration from a file and saves it in a hashtable
    * @exception IOException if an error occures during reading,
    * e.g. because it was not a
    * valid configuration file, the file doesnt
    * exists or the user has no reading permissions etc.
    * @param configurationFile configuration file
    * @return configuration saved in a Hashtable.
    * Access indices are the action numbers.
    * Elements are objects of class ActionConfigurations.
    * @see ActionConfiguration
    */
    private static Hashtable readConfiguration(File configurationFile) throws IOException {
	try {
	    // does the file exists, can it be read?
	    if(configurationFile == null || !configurationFile.canRead())
		throw new IOException();

	    // determine file length
	    long fileLengh = configurationFile.length();
	    // is it a valid file?
	    if((fileLengh % ActionConfiguration.STREAM_SIZE) != 0)
		throw new IOException();

	    FileInputStream fileInputStream = new FileInputStream(configurationFile);
	    Hashtable hashTable = new Hashtable();

	    //for every saved action configuration
	    for(int i = 0; i < (fileLengh / ActionConfiguration.STREAM_SIZE); i++) {
		ActionConfiguration actionConfiguration = ActionConfiguration.fromStream(fileInputStream);
		//save in Hashtable
		hashTable.put(new Integer(actionConfiguration.getID()), actionConfiguration);
	    }
	    return hashTable;
	} catch(Exception e) {
	    throw new IOException();
	}
    }

    /** Writes configuration data from Hashtable to configuration file
    * The writing is actually done by ActionConfiguration.toStream
    * @param configurationFile configuration file. If there is already a file, it will be overwritten.
    * Otherwise a new file will be created.
    * @param hashTable Hashtable containing configuration data
    * @exception IOException if an error occurs while writing
    */
    private static void writeConfiguration(File configurationFile, Hashtable hashTable) throws IOException {
	try {
	    FileOutputStream fileOutputStream = new FileOutputStream(configurationFile);
	    // walk through the elements of the Hashtable
	    for (Enumeration enumeration = hashTable.elements(); enumeration.hasMoreElements();)
		// and write to file
		((ActionConfiguration)enumeration.nextElement()).toStream(fileOutputStream, false);
	    fileOutputStream.close();
	} catch(Exception e) {
	    throw new IOException();
	}
    }


    /** Creates the dialog. Four panels will be created and displayed, one for each security goal.
    * Then the dialog waits for user inputs. A click on cancel will close the dialog without taking over
    * the changes made. A click on OK ends the Dialog and saves the changes into the configuration file
    * @param parentFrame Neccessary for a modal dialog
    * @param applicationName Name of the application. Is only used to determine the name of the configuration file
    * @param actionName Shows up in the Titlebar of the window. No other meaning...
    * @param actionID Unique action number. Configuration data will be saved under this index.
    * @param actionConstraint security settings.
    */
    private ApplicationConfiguration(JFrame parentFrame,
	String applicationName,
	String actionName,
	short actionID,
	SSONETConstraints actionConstraint) {

	super(parentFrame, actionName, true);

	ActionConfiguration defaultValues = new ActionConfiguration (
	    actionID,
	    actionConstraint.def[0], actionConstraint.def[1],
	    actionConstraint.def[2], actionConstraint.def[3]);
	ActionConfiguration minimalSecuritySettings = new ActionConfiguration (
	    actionID,
	    actionConstraint.min[0], actionConstraint.min[1],
	    actionConstraint.min[2], actionConstraint.min[3]);
	ActionConfiguration maximalSecuritySettings = new ActionConfiguration (
	    actionID,
	    actionConstraint.max[0], actionConstraint.max[1],
	    actionConstraint.max[2], actionConstraint.max[3]);

	interactiveTime=System.currentTimeMillis();
	this.actionID = actionID;
	this.applicationName = applicationName;
	this.defaultValues = defaultValues;
	configuration = null;

	// load Configuration
	configurationFile = getConfigurationFile(applicationName);
	try {
	    configuration = readConfiguration(configurationFile);
	} catch (Exception e) {
	    System.out.println("Error while reading configuration");
	    e.printStackTrace();
	}

	// if not configuration existent --> create empty configuration
	if(configuration == null)
	    configuration = new Hashtable();

	// search configuration for action
	actionConfiguration = (ActionConfiguration)configuration.get(new Integer(actionID));

	// use default if no configuration data existent
	if(actionConfiguration == null) actionConfiguration = defaultValues;

	// panel which contains the four panels representing the security goals
	JPanel mainPanel = new JPanel();
	mainPanel.setLayout(new GridLayout(2,2,3,3));

	// create panel Confidentiality
	panelConfidentiality = new ApplicationConfigurationPanel
	    ("Confidentiality",
	    new Color(192, 210, 192),
	    "images/vertraulich.gif",
	    "images/nichtvertraulich.gif",
	    "images/egalvertraulich.gif",
	    actionConfiguration.getSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY),
	    minimalSecuritySettings.getSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY),
	    maximalSecuritySettings.getSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY));

	// create panel Anonymity
	panelAnonymity = new ApplicationConfigurationPanel
	    ("Anonymity",
	    new Color(192,210,192),
	    "images/anonym.gif",
	    "images/nichtanonym.gif",
	    "images/egalanonym.gif",
	    actionConfiguration.getSecurityGoalLevel(SSONETConstants.ANONYMITY),
	    minimalSecuritySettings.getSecurityGoalLevel(SSONETConstants.ANONYMITY),
	    maximalSecuritySettings.getSecurityGoalLevel(SSONETConstants.ANONYMITY));

	// create panel Integrity
	panelIntegrity = new ApplicationConfigurationPanel
	    ("Integrity",
	    new Color(192,192,210),
	    "images/integer.gif",
	    "images/nichtinteger.gif",
	    "images/egalinteger.gif",
	    actionConfiguration.getSecurityGoalLevel(SSONETConstants.INTEGRITY),
	    minimalSecuritySettings.getSecurityGoalLevel(SSONETConstants.INTEGRITY),
	    maximalSecuritySettings.getSecurityGoalLevel(SSONETConstants.INTEGRITY));

	// create panel Accountability
	panelAccountability = new ApplicationConfigurationPanel
	    ("Accountability",
	    new Color(192,192,210),
	    "images/zurechenbar.gif",
	    "images/nichtzurechenbar.gif",
	    "images/egalzurechenbar.gif",
	    actionConfiguration.getSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN),
	    minimalSecuritySettings.getSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN),
	    maximalSecuritySettings.getSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN));
	// add Panels
	mainPanel.add(panelConfidentiality);
	mainPanel.add(panelAnonymity);
	mainPanel.add(panelIntegrity);
	mainPanel.add(panelAccountability);

	// parting lines
	mainPanel.setBackground(Color.black);

	getContentPane().add("Center", mainPanel);

	// Panel for Buttons
	JPanel buttonPanel = new JPanel();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	GridBagLayout gridBagLayout = new GridBagLayout();

	gridBagConstraints.insets = new Insets(10,10,10,10);
	gridBagConstraints.anchor = GridBagConstraints.CENTER;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagConstraints.ipadx = 0;
	gridBagConstraints.ipady = 0;
	gridBagConstraints.weightx = 0.333;
	gridBagConstraints.weighty = 1;

	buttonPanel.setLayout(gridBagLayout);
	okButton = new JButton("     Ok     ");
	gridBagLayout.setConstraints(okButton, gridBagConstraints);
	buttonPanel.add(okButton);
	okButton.addActionListener(this);

	gridBagConstraints.gridx = 1;
	JButton cancelButton = new JButton(" Cancel ");
	gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
	buttonPanel.add(cancelButton);
	cancelButton.addActionListener(this);

	gridBagConstraints.gridx = 2;
	defaultButton = new JButton(" Default ");
	gridBagLayout.setConstraints(defaultButton, gridBagConstraints);
	buttonPanel.add(defaultButton);
	defaultButton.addActionListener(this);

	gridBagConstraints.gridx = 3;
	helpButton = new JButton(" Help ");
	gridBagLayout.setConstraints(helpButton, gridBagConstraints);
	buttonPanel.add(helpButton);
	helpButton.addActionListener(this);

	getContentPane().add("South", buttonPanel);

	pack();
	setResizable(false);

	// show dialog centered
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension ownDimension = getSize();
	setLocation((screenSize.width - ownDimension.width) / 2,
            (screenSize.height - ownDimension.height) / 2);
    }


    /*** This method creates the configuration dialog and loads saved values from the configuration database.
    * When the dialog for a action is called the first time default values are used
    * @param parentFrame for modal dialog neccessary
    * @param applicationName Name of the application
    * @param actionName Shows up in the Titlebar of the window. No other meaning...
    * @param actionID unique action number
    * @param configuration security settings
    */
    final public static void configureAction(JFrame parentFrame,
					     String applicationName,
					     String actionName,
					     short actionID,
					     SSONETConstraints configuration) {
	ApplicationConfiguration applicationConfiguration = new ApplicationConfiguration(parentFrame,
	    applicationName,
	    actionName,
	    actionID,
	    configuration);
	applicationConfiguration.show();
    }



    /** returns the configuration for a given action
    * @param applicationName name of the application. Name of the configuration file is : applicationName + '.cfg'
    * @param actionID unique action number
    * @return action settings
    * null - if no configuration settings available or an error occured
    */
    final public static ActionConfiguration getActionConfiguration(String applicationName, int actionID) {
	try {
	    Hashtable configuration = null;
	    configuration = readConfiguration(getConfigurationFile(applicationName));
	    return (ActionConfiguration)configuration.get(new Integer(actionID));
	} catch (Exception e) {
	    System.out.println("Error while reading configuration.");
	    e.printStackTrace();
	    return null;
	}
    }

    /** handles Actionlistener calls
    */
    final public void actionPerformed(ActionEvent actionEvent) {
	if(actionEvent.getSource() == okButton) {
	    // read values from panels
	    actionConfiguration.setSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY, panelConfidentiality.getState());
	    actionConfiguration.setSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN, panelAccountability.getState());
	    actionConfiguration.setSecurityGoalLevel(SSONETConstants.INTEGRITY, panelIntegrity.getState());
	    actionConfiguration.setSecurityGoalLevel(SSONETConstants.ANONYMITY, panelAnonymity.getState());

	    // put configuration in Hasthtable
	    configuration.put(new Integer(actionID), actionConfiguration);
	    try {
		// save configuration
		writeConfiguration(configurationFile, configuration);
	    }
	    catch (Exception ex) {
		ex.printStackTrace();
	    }
	    interactiveTime=System.currentTimeMillis()-interactiveTime;
	    dispose();
	}
	else if(actionEvent.getSource() == helpButton) {

	    interactiveTime=System.currentTimeMillis()-interactiveTime;
	    Help hp = new Help("file:///"+System.getProperty("ssonet_help")+"gk/HelpSet.hs");
	    hp.viewPage("ACTIONCONFIGURATION","");
	}
	else if(actionEvent.getSource() == defaultButton) {

	    interactiveTime=System.currentTimeMillis()-interactiveTime;
	    panelConfidentiality.setState(
		defaultValues.getSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY));
	    panelAccountability.setState(
		defaultValues.getSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN));
	    panelAnonymity.setState(
		defaultValues.getSecurityGoalLevel(SSONETConstants.ANONYMITY));
	    panelIntegrity.setState(
		defaultValues.getSecurityGoalLevel(SSONETConstants.INTEGRITY));

	}
	else {

	    interactiveTime=System.currentTimeMillis()-interactiveTime;
	    dispose();
	}
    }


    final public static long getInteractiveTime() {

	return interactiveTime;
    }
}
