/*
 * smi/appConf/ApplicationConfigurationPanel.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.appConf;

import java.awt.Choice;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/** This class represents the input area for a security goal.
* It displays e.g. the checkboxes 'always' - 'never'.
*/
final class ApplicationConfigurationPanel extends JPanel implements ItemListener {
    /** combobox for choosing security goal settings
    */
    private Choice comboBox;

    /** checkbox for 'negotiable'
    */
    private JCheckBox checkBox;
    private final static int INDEX[] = {4,4,3,2,1,0,0};
    private JLabel icon;
    private ImageIcon iconOn, iconOff, iconEqual;
    private byte minimalSecuritySettings, maximalSecuritySettings;

    /** creates a panels for a security goal
    * @param securityGoalName Name of the security goal
    * @param backgroundColor background color
    * @param iconOnFileName name of the file containing the picture when the security goal is "on"
    * @param iconOffFileName name of the file containing the picture when the security goal is "off"
    * @param iconEqualFileName name of the file containing the picture when the security goal "doesnt matter"
    * @param initialSecuritySetting initial security setting
    * @param minimalSecuritySettings determines the minimal security settings which a user can not fall short of
    * @param maximalSecuritySettings determines the maximal security settings which a user can not exceed
    */
    ApplicationConfigurationPanel(String securityGoalName,
	Color backgroundColor,
	String iconOnFileName,
	String iconOffFileName,
	String iconEqualFileName,
	byte initialSecuritySetting,
	byte minimalSecuritySettings,
	byte maximalSecuritySettings)
    {
	super();
	this.minimalSecuritySettings = minimalSecuritySettings;
	this.maximalSecuritySettings = maximalSecuritySettings;
	iconOn = new ImageIcon(getClass().getResource(iconOnFileName));
	iconOff = new ImageIcon(getClass().getResource(iconOffFileName));
	iconEqual = new ImageIcon(getClass().getResource(iconEqualFileName));

	setBackground(backgroundColor);
	String[] strCheckboxItems = {
	    "unconditional",
		"if possible",
		"don't care",
		"if neccessary",
		"on no condition"
	};
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	GridBagLayout gridBagLayout = new GridBagLayout();
	gridBagConstraints.insets = new Insets(0,0,0,0);
	gridBagConstraints.anchor = GridBagConstraints.CENTER;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 2;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagConstraints.ipadx = 0;
	gridBagConstraints.ipady = 0;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = 0;
	JLabel securityGoalLabel = new JLabel(securityGoalName);
	securityGoalLabel.setFont(new Font("Monospaced", Font.BOLD,14));
	gridBagLayout.setConstraints(securityGoalLabel, gridBagConstraints);
	add(securityGoalLabel);

	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.gridx = 1;
	gridBagConstraints.gridy = 1;
	gridBagConstraints.insets = new Insets(10,10,10,10);
	comboBox = new Choice();
	for(int i = INDEX[maximalSecuritySettings]; i <= INDEX[minimalSecuritySettings]; i++)
	    comboBox.addItem(strCheckboxItems[i]);
	comboBox.setFont(new Font("Monospaced", Font.PLAIN,12));
	gridBagLayout.setConstraints(comboBox, gridBagConstraints);
	comboBox.addItemListener(this);
	add(comboBox);
	gridBagConstraints.gridx = 1;
	gridBagConstraints.gridy = 2;
	gridBagConstraints.insets = new Insets(0,20,0,0);
	checkBox = new JCheckBox("negotiable", false);
	if((initialSecuritySetting == 1) || (initialSecuritySetting == 5))
	    checkBox.setSelected(true);
	else if((initialSecuritySetting > 1) && (initialSecuritySetting < 5))
	    checkBox.setEnabled(false);

	gridBagLayout.setConstraints(checkBox, gridBagConstraints);
	add(checkBox);

	gridBagConstraints.insets = new Insets(10,10,10,10);
	gridBagConstraints.gridy = 1;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridheight = 2;
	gridBagConstraints.anchor = GridBagConstraints.CENTER;

	if(initialSecuritySetting > 3) icon = new JLabel(iconOn);
	else if(initialSecuritySetting < 3) icon = new JLabel(iconOff);
	else icon = new JLabel(iconEqual);
	iconOff.setImageObserver(icon);
	gridBagLayout.setConstraints(icon, gridBagConstraints);
	add(icon);

	setLayout(gridBagLayout);
	comboBox.select(INDEX[initialSecuritySetting]-INDEX[maximalSecuritySettings]);
    }

    final public void itemStateChanged(ItemEvent e) {
	if(e.getStateChange() == ItemEvent.SELECTED) {
	    int i = comboBox.getSelectedIndex() + INDEX[maximalSecuritySettings];
	    setIcon(i);
	}
    }

    void setIcon(int i) {
	if(i == 0) {
	    checkBox.setEnabled(true);
	    icon.setIcon(iconOn);
	} else if(i == 4) {
	    checkBox.setEnabled(true);
	    icon.setIcon(iconOff);
	} else {
	    checkBox.setEnabled(false);
	    checkBox.setSelected(false);
	    if (i == 1)
		icon.setIcon(iconOn);
	    else if(i == 3)
		icon.setIcon(iconOff);
	    else
		icon.setIcon(iconEqual);
	}
    }

    /** returns chosen settings for the security goal
    * @return settings for the security goal
    */
    final public byte getState() {
	int i = comboBox.getSelectedIndex() + INDEX[maximalSecuritySettings];
	if(i == 0) {
	    i = 6;
	    if(checkBox.isSelected()) i--;
	} else if(i == 4) {
	    if(checkBox.isSelected()) i = 1;
	    else i = 0;
	} else
	    i = 5 - i;
	return (byte)i;
    }
    final public void setState(byte state) {
	comboBox.select(INDEX[state]-INDEX[maximalSecuritySettings]);
	setIcon(comboBox.getSelectedIndex() + INDEX[maximalSecuritySettings]);
    }
}
